/****************************************************************************************
***** Macro qui assigne l'indice de dfavorisation de 2011 du Qubec ainsi que ses 	*****
***** dclinaisons locales (RSS, RLS, CLSC) selon les dcoupages pr-rforme 2015. 	*****
***** L'assignation est ralise  partir du code postal (obligatoire) et du code	*****
***** municipal (facultatif). La macro est base sur une version simplifie du		*****
***** Fichier de conversion des codes postaux (FCCP+) de Statistique Canada et fait *****
***** appel aux trois fichiers de conversion suivants:								*****
*****  - Fichier de codes postaux uniques (PCCFUNIQ)                                *****
*****  - Fichier de codes postaux non uniques (PCCFDUPS)                           	*****
*****  - Fichier de codes postaux non uniques avec pondration connue (WCF)         *****
*****  cela s'ajoute un quatrime fichier (MUNIC) qui effectue l'historique des	*****
***** codes municipaux au Canada de 1991  2011.									*****
*****																				***** 
***** Les quatre paramtres de la macro sont:                                       *****
*****  - IN: nom du fichier SAS en entre auquel on dsire ajouter l'indice de   	*****
*****        dfavorisation de 2011.						    					*****
*****  - PCODE: nom de la variable du code postal dans le fichier d'entre.    		*****
*****  - MUNIC: nom de la variable du code municipal dans le fichier d'entre. Par  *****
*****           dfaut, la valeur 0 (zro) signifie que l'assignation est ralise  *****
*****           seulement  partir du code postal.                        			*****
*****  - OUT: nom du fichier SAS de sortie, qui sera identique au fichier d'entre 	*****
*****         mais bonifi des deux nouvelles variables.      						*****
*****																				*****
***** Auteur: Denis Hamel, INSPQ                                    				*****
*****         Philippe Gamache, INSPQ                               				*****
***** Dernire modification: 2 septembre 2015                        				*****
*****************************************************************************************/


/****************************************************************************************/
/* 1. tablir le chemin daccs du rpertoire o se trouve les 4 fichiers de donnes    */
%let repertoire = D:\Projets;


/****************************************************************************************/
/* 2. Dterminer les quatre paramtres (par dfaut, munic = 0)						    */
%let in = fichier_depart;
%let pcode = codepostal;
%let munic = 0;
%let out = fichier_assigne;


/****************************************************************************************/
/* 3. Excution de la macro. Ne rien changer  partir d'ici						    	*/
%macro assignation2011(in=, pcode=, munic=, out=);

filename fcp "&repertoire\fcp.txt";
filename pccfuniq "&repertoire\fccpindiceuniq.txt";
filename pccfdups "&repertoire\fccpindicedouble.txt";
filename munic "&repertoire\munic2011.txt";

libname i2011 "&repertoire";

data i2011.wcf;
  infile fcp;
  input munic 1-12 pcode $ 13-18 propmunic 19-23 .2 propcp 5.2 first 29 QuintMat 30 QuintSoc 31 QuintMatRSS 32 QuintSocRSS 33 QuartMatRSS 34 QuartSocRSS 35 
       QuintMatRTS 36 QuintSocRTS 37 QuartMatRTS 38 QuartSocRTS 39 QuartMatRLS 40 QuartSocRLS 41 QuartMatCLSC 42 QuartSocCLSC 43 zone 44 RSS $ 45-46 RTS $ 47-49 
       RLS $ 50-53 CLSC $ 54-58 discord 59  TGN_AD 5.1 propmunic_RLS 5.2 propcp_RLS 5.2 propmunic_CLSC 5.2 propcp_CLSC 5.2;
run;

data i2011.pccfuniq;
  infile pccfuniq;
  input munic 1-12 pcode $ 13-18 propmunic 19-23 .2 propcp 5.2 first 29 QuintMat 30 QuintSoc 31 QuintMatRSS 32 QuintSocRSS 33 QuartMatRSS 34 QuartSocRSS 35 
       QuintMatRTS 36 QuintSocRTS 37 QuartMatRTS 38 QuartSocRTS 39 QuartMatRLS 40 QuartSocRLS 41 QuartMatCLSC 42 QuartSocCLSC 43 zone 44 RSS $ 45-46 RTS $ 47-49 
       RLS $ 50-53 CLSC $ 54-58 discord 59  TGN_AD 5.1 propmunic_RLS 5.2 propcp_RLS 5.2 propmunic_CLSC 5.2 propcp_CLSC 5.2;
run;

data i2011.pccfdups;
  infile pccfdups;
  input munic 1-12 pcode $ 13-18 propmunic 19-23 .2 propcp 5.2 first 29 QuintMat 30 QuintSoc 31 QuintMatRSS 32 QuintSocRSS 33 QuartMatRSS 34 QuartSocRSS 35 
       QuintMatRTS 36 QuintSocRTS 37 QuartMatRTS 38 QuartSocRTS 39 QuartMatRLS 40 QuartSocRLS 41 QuartMatCLSC 42 QuartSocCLSC 43 zone 44 RSS $ 45-46 RTS $ 47-49 
       RLS $ 50-53 CLSC $ 54-58 discord 59  TGN_AD 5.1 propmunic_RLS 5.2 propcp_RLS 5.2 propmunic_CLSC 5.2 propcp_CLSC 5.2;
run;

data i2011.munic;
  infile munic;
  input munic 1-7 mun2011 8-14;
run;

data wcf;
   set i2011.wcf;
   nuniq = _n_;
run;

data pccfuniq;
   set i2011.pccfuniq;
   nuniq = _n_;
run;

data pccfdups;
   set i2011.pccfdups;
   nuniq = _n_;
run;


data fichin;
  %if &munic ne 0 %then %do; 
    length munic 8.; 
  %end;
  set &in;
  numero=_n_;
  nba=ranuni(766);
  pcode=compress(&pcode);
  %if &munic ne 0 %then %do; 
    munic=2400000+mod(&munic,2400000); 
  %end;
run;

%presence(var=quintmat);

%if &presence ne 0 %then %do;
data fichin;
  set fichin;
  where zone in (.,0);
run;
%end;

data fichin;
  set fichin;
  keep numero nba pcode %if &munic ne 0 %then %do; munic %end;;
run;


/****************************************************************************************/
/* Assignation de l'indice  partir du code postal et de la municipalit (si munic ne 0)*/

%if &munic ne 0 %then %do;

proc sort data=i2011.munic;
  by munic;
run;

proc sort data=fichin;
  by munic;
run;

data fichin;
  merge i2011.munic(in=in2) fichin(in=in1);
  by munic;
  if in1*in2 then munic=mun2011;
  else if in2 then delete;
run;

/***** Avec PCCFUNIQ *****/
/*** RLS ***/
proc sql;
  create table test as
  select fichin.pcode, fichin.munic, fichin.numero, fichin.nba, pccfuniq.first, pccfuniq.quartmatRLS, pccfuniq.quartsocRLS, 
         pccfuniq.munic, pccfuniq.propmunic_RLS, pccfuniq.nuniq, pccfuniq.RLS
  from fichin, pccfuniq
  where fichin.pcode=pccfuniq.pcode and fichin.munic=pccfuniq.munic
  order by numero,nuniq;
quit;

proc sort data=test;
  by pcode munic numero;
run;

 /*** Choix de l'indice de dfavorisation si plus d'un indice par code postal ***/
data test1;
  set test;
  by pcode munic numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1; 
    else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propmunic_RLS/100);
	if nba < cumul then do; 
      find=1; output; 
    end;
  end;
run; 

/*** CLSC ***/
proc sql;
  create table test as
  select fichin.pcode, fichin.munic, fichin.numero, fichin.nba, pccfuniq.first, pccfuniq.quartmatCLSC, pccfuniq.quartsocCLSC, 
         pccfuniq.munic, pccfuniq.propmunic_CLSC, pccfuniq.nuniq, pccfuniq.CLSC 
  from fichin, pccfuniq
  where fichin.pcode=pccfuniq.pcode and fichin.munic=pccfuniq.munic
  order by numero,nuniq;
quit;

proc sort data=test;
  by pcode munic numero;
run;

 /*** Choix de l'indice de dfavorisation si plus d'un indice par code postal ***/
data test2;
  set test;
  by pcode munic numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1; 
    else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propmunic_CLSC/100);
	if nba < cumul then do; 
      find=1; output; 
    end;
  end;
run; 

/*** Qubec et RSS ***/
proc sql;
  create table test as
  select fichin.pcode, fichin.munic, fichin.numero, fichin.nba, pccfuniq.first, pccfuniq.quintmat, pccfuniq.quintsoc, pccfuniq.quintmatRSS, pccfuniq.quintsocRSS, 
         pccfuniq.quartmatRSS, pccfuniq.quartsocRSS, pccfuniq.munic, pccfuniq.propmunic, pccfuniq.nuniq, pccfuniq.zone, pccfuniq.RSS, pccfuniq.RTS, 
		 pccfuniq.TGN_AD, pccfuniq.discord, pccfuniq.quintmatRTS, pccfuniq.quintsocRTS, pccfuniq.quartmatRTS, pccfuniq.quartsocRTS
  from fichin, pccfuniq
  where fichin.pcode=pccfuniq.pcode and fichin.munic=pccfuniq.munic
  order by numero,nuniq;
quit;

proc sort data=test;
  by pcode munic numero;
run;

 /*** Choix de l'indice de dfavorisation si plus d'un indice par code postal ***/
data test3;
  set test;
  by pcode munic numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1; 
    else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propmunic/100);
	if nba < cumul then do; 
      find=1; output; 
    end;
  end;
run; 

proc sort data=test1;
  by numero;
run;

proc sort data=test2;
  by numero;
run;

proc sort data=test3;
  by numero;
run;

proc sort data=fichin;
  by numero;
run; 

data appmunic1 nonapp1;
  merge fichin(in=in1) test1(in=in2a) test2(in=in2b) test3(in=in2c);
  by numero;
  if in1*in2a*in2b*in2c then output appmunic1;
  else if in1 then output nonapp1;
run;

/***** Avec WCF *****/
/*** RLS ***/
proc sql;
  create table test as
  select fichin.pcode, fichin.munic, fichin.numero, fichin.nba, wcf.first, wcf.quartmatRLS, wcf.quartsocRLS, 
         wcf.munic, wcf.propmunic_RLS, wcf.nuniq, wcf.RLS 
  from fichin, wcf
  where fichin.pcode=wcf.pcode and fichin.munic=wcf.munic
  order by numero,nuniq;
quit;

proc sort data=test;
  by pcode munic numero;
run;

 /*** Choix de l'indice de dfavorisation si plus d'un indice par code postal ***/
data test1;
  set test;
  by pcode munic numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1; 
    else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propmunic_RLS/100);
	if nba < cumul then do; 
      find=1; output; 
    end;
  end;
run; 

/*** CLSC ***/
proc sql;
  create table test as
  select fichin.pcode, fichin.munic, fichin.numero, fichin.nba, wcf.first, wcf.quartmatCLSC, wcf.quartsocCLSC, 
         wcf.munic, wcf.propmunic_CLSC, wcf.nuniq, wcf.CLSC 
  from fichin, wcf
  where fichin.pcode=wcf.pcode and fichin.munic=wcf.munic
  order by numero,nuniq;
quit;

proc sort data=test;
  by pcode munic numero;
run;

 /*** Choix de l'indice de dfavorisation si plus d'un indice par code postal ***/
data test2;
  set test;
  by pcode munic numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1; 
    else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propmunic_CLSC/100);
	if nba < cumul then do; 
      find=1; output; 
    end;
  end;
run; 

/*** Qubec et RSS ***/
proc sql;
  create table test as
  select fichin.pcode, fichin.munic, fichin.numero, fichin.nba, wcf.first, wcf.quintmat, wcf.quintsoc, wcf.quintmatRSS, wcf.quintsocRSS, 
         wcf.quartmatRSS, wcf.quartsocRSS, wcf.munic, wcf.propmunic, wcf.nuniq, wcf.RSS, wcf.RTS, wcf.TGN_AD, wcf.discord, wcf.zone,
		 wcf.quintmatRTS, wcf.quintsocRTS, wcf.quartmatRTS, wcf.quartsocRTS
  from fichin, wcf
  where fichin.pcode=wcf.pcode and fichin.munic=wcf.munic
  order by numero,nuniq;
quit;

proc sort data=test;
  by pcode munic numero;
run;

 /*** Choix de l'indice de dfavorisation si plus d'un indice par code postal ***/
data test3;
  set test;
  by pcode munic numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1; 
    else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propmunic/100);
	if nba < cumul then do; 
      find=1; output; 
    end;
  end;
run; 

proc sort data=test1;
  by numero;
run;

proc sort data=test2;
  by numero;
run;

proc sort data=test3;
  by numero;
run;

proc sort data=nonapp1;
  by numero;
run; 

data appmunic2 nonapp2;
  merge nonapp1(in=in1) test1(in=in2a) test2(in=in2b) test3(in=in2c);
  by numero;
  if in1*in2a*in2b*in2c then output appmunic2;
  else if in1 then output nonapp2;
run;

/***** Avec PCCFDUPS *****/
/*** RLS ***/
proc sql;
  create table test as
  select fichin.pcode, fichin.munic, fichin.numero, fichin.nba, pccfdups.first, pccfdups.quartmatRLS, pccfdups.quartsocRLS, 
         pccfdups.munic, pccfdups.propmunic_RLS, pccfdups.nuniq, pccfdups.RLS 
  from fichin, pccfdups
  where fichin.pcode=pccfdups.pcode and fichin.munic=pccfdups.munic
  order by numero,nuniq;
quit;

proc sort data=test;
  by pcode munic numero;
run;

 /*** Choix de l'indice de dfavorisation si plus d'un indice par code postal ***/
data test1;
  set test;
  by pcode munic numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1; 
    else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propmunic_RLS/100);
	if nba < cumul then do; 
      find=1; output; 
    end;
  end;
run; 

/*** CLSC ***/
proc sql;
  create table test as
  select fichin.pcode, fichin.munic, fichin.numero, fichin.nba, pccfdups.first, pccfdups.quartmatCLSC, pccfdups.quartsocCLSC, 
         pccfdups.munic, pccfdups.propmunic_CLSC, pccfdups.nuniq, pccfdups.CLSC 
  from fichin, pccfdups
  where fichin.pcode=pccfdups.pcode and fichin.munic=pccfdups.munic
  order by numero,nuniq;
quit;

proc sort data=test;
  by pcode munic numero;
run;

 /*** Choix de l'indice de dfavorisation si plus d'un indice par code postal ***/
data test2;
  set test;
  by pcode munic numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1; 
    else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propmunic_CLSC/100);
	if nba < cumul then do; 
      find=1; output; 
    end;
  end;
run; 

/*** Qubec et RSS ***/
proc sql;
  create table test as
  select fichin.pcode, fichin.munic, fichin.numero, fichin.nba, pccfdups.first, pccfdups.quintmat, pccfdups.quintsoc, pccfdups.quintmatRSS, pccfdups.quintsocRSS, 
         pccfdups.quartmatRSS, pccfdups.quartsocRSS, pccfdups.munic, pccfdups.propmunic, pccfdups.nuniq, pccfdups.zone, pccfdups.RSS, pccfdups.RTS, 
         pccfdups.TGN_AD, pccfdups.discord, pccfdups.quintmatRTS, pccfdups.quintsocRTS, pccfdups.quartmatRTS, pccfdups.quartsocRTS
  from fichin, pccfdups
  where fichin.pcode=pccfdups.pcode and fichin.munic=pccfdups.munic
  order by numero,nuniq;
quit;

proc sort data=test;
  by pcode munic numero;
run;

 /*** Choix de l'indice de dfavorisation si plus d'un indice par code postal ***/
data test3;
  set test;
  by pcode munic numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1; 
    else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propmunic/100);
	if nba < cumul then do; 
      find=1; output; 
    end;
  end;
run; 

proc sort data=test1;
  by numero;
run;

proc sort data=test2;
  by numero;
run;

proc sort data=test3;
  by numero;
run;

proc sort data=nonapp2;
  by numero;
run; 

data appmunic3 fichin;
  merge nonapp2(in=in1) test1(in=in2a) test2(in=in2b) test3(in=in2c);
  by numero;
  if in1*in2a*in2b*in2c then output appmunic3;
  else if in1 then output fichin;
run;


%end;

/****************************************************************************************/
/* Assignation de l'indice  partir du code postal seulement							*/

/***** Avec PCCFUNIQ *****/
/*** RLS ***/
proc sql;
  create table test as
  select fichin.pcode, fichin.numero, fichin.nba, pccfuniq.first, pccfuniq.quartmatRLS, pccfuniq.quartsocRLS, 
         pccfuniq.propcp_RLS, pccfuniq.nuniq, pccfuniq.RLS 
  from fichin, pccfuniq
  where fichin.pcode=pccfuniq.pcode
  order by numero,nuniq;
quit;

proc sort data=test;
  by pcode numero;
run;

 /*** Choix de l'indice de dfavorisation si plus d'un indice par code postal ***/
data test1;
  set test;
  by pcode numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1; 
    else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propcp_RLS/100);
	if nba < cumul then do; 
      find=1; output; 
    end;
  end;
run; 

/*** CLSC ***/
proc sql;
  create table test as
  select fichin.pcode, fichin.numero, fichin.nba, pccfuniq.first, pccfuniq.quartmatCLSC, pccfuniq.quartsocCLSC, 
         pccfuniq.propcp_CLSC, pccfuniq.nuniq, pccfuniq.CLSC 
  from fichin, pccfuniq
  where fichin.pcode=pccfuniq.pcode
  order by numero,nuniq;
quit;

proc sort data=test;
  by pcode numero;
run;

 /*** Choix de l'indice de dfavorisation si plus d'un indice par code postal ***/
data test2;
  set test;
  by pcode numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1; 
    else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propcp_CLSC/100);
	if nba < cumul then do; 
      find=1; output; 
    end;
  end;
run; 

/*** Qubec et RSS ***/
proc sql;
  create table test as
  select fichin.pcode, fichin.numero, fichin.nba, pccfuniq.first, pccfuniq.quintmat, pccfuniq.quintsoc, pccfuniq.quintmatRSS, pccfuniq.quintsocRSS, 
         pccfuniq.quartmatRSS, pccfuniq.quartsocRSS, pccfuniq.propcp, pccfuniq.nuniq, pccfuniq.zone, pccfuniq.RSS, pccfuniq.RTS, pccfuniq.TGN_AD, pccfuniq.discord,
		 pccfuniq.quintmatRTS, pccfuniq.quintsocRTS, pccfuniq.quartmatRTS, pccfuniq.quartsocRTS
  from fichin, pccfuniq
  where fichin.pcode=pccfuniq.pcode
  order by numero,nuniq;
quit;

proc sort data=test;
  by pcode numero;
run;

/*** Choix de l'indice de dfavorisation si plus d'un indice par code postal ***/
data test3;
  set test;
  by pcode numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1; 
    else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propcp/100);
	if nba < cumul then do; 
      find=1; output; 
    end;
  end;
run; 

proc sort data=test1;
  by numero;
run;

proc sort data=test2;
  by numero;
run;

proc sort data=test3;
  by numero;
run;

proc sort data=fichin;
  by numero;
run; 

data appcp1 nonapp3;
  merge fichin(in=in1) test1(in=in2a) test2(in=in2b) test3(in=in2c);
  by numero;
  if in1*in2a*in2b*in2c then output appcp1;
  else if in1 then output nonapp3;
run;

/***** Avec WCF *****/
/*** RLS ***/
proc sql;
  create table test as
  select fichin.pcode, fichin.numero, fichin.nba, wcf.first, wcf.quartmatRLS, wcf.quartsocRLS, 
         wcf.propcp_RLS, wcf.nuniq, wcf.RLS 
  from fichin, wcf
  where fichin.pcode=wcf.pcode
  order by numero,nuniq;
quit;

proc sort data=test;
  by pcode numero;
run;

 /*** Choix de l'indice de dfavorisation si plus d'un indice par code postal ***/
data test1;
  set test;
  by pcode numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1; 
    else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propcp_RLS/100);
	if nba < cumul then do; 
      find=1; output; 
    end;
  end;
run; 

/*** CLSC ***/
proc sql;
  create table test as
  select fichin.pcode, fichin.numero, fichin.nba, wcf.first, wcf.quartmatCLSC, wcf.quartsocCLSC, 
         wcf.propcp_CLSC, wcf.nuniq, wcf.CLSC 
  from fichin, wcf
  where fichin.pcode=wcf.pcode
  order by numero,nuniq;
quit;

proc sort data=test;
  by pcode numero;
run;

 /*** Choix de l'indice de dfavorisation si plus d'un indice par code postal ***/
data test2;
  set test;
  by pcode numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1; 
    else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propcp_CLSC/100);
	if nba < cumul then do; 
      find=1; output; 
    end;
  end;
run; 

/*** Qubec et RSS ***/
proc sql;
  create table test as
  select fichin.pcode, fichin.numero, fichin.nba, wcf.first, wcf.quintmat, wcf.quintsoc, wcf.quintmatRSS, wcf.quintsocRSS, 
         wcf.quartmatRSS, wcf.quartsocRSS, wcf.propcp, wcf.nuniq, wcf.zone, wcf.RSS, wcf.RTS, wcf.TGN_AD, wcf.discord,
		 wcf.quintmatRTS, wcf.quintsocRTS, wcf.quartmatRTS, wcf.quartsocRTS
  from fichin, wcf
  where fichin.pcode=wcf.pcode
  order by numero,nuniq;
quit;

proc sort data=test;
  by pcode numero;
run;

/*** Choix de l'indice de dfavorisation si plus d'un indice par code postal ***/
data test3;
  set test;
  by pcode numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1; 
    else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propcp/100);
	if nba < cumul then do; 
      find=1; output; 
    end;
  end;
run; 

proc sort data=test1;
  by numero;
run;

proc sort data=test2;
  by numero;
run;

proc sort data=test3;
  by numero;
run;

proc sort data=nonapp3;
  by numero;
run; 

data appcp2 nonapp4;
  merge nonapp3(in=in1) test1(in=in2a) test2(in=in2b) test3(in=in2c);
  by numero;
  if in1*in2a*in2b*in2c then output appcp2;
  else if in1 then output nonapp4;
run;

/***** Avec PCCFDUPS *****/
/*** RLS ***/
proc sql;
  create table test as
  select fichin.pcode, fichin.numero, fichin.nba, pccfdups.first, pccfdups.quartmatRLS, pccfdups.quartsocRLS, 
         pccfdups.propcp_RLS, pccfdups.nuniq, pccfdups.RLS 
  from fichin, pccfdups
  where fichin.pcode=pccfdups.pcode
  order by numero,nuniq;
quit;

proc sort data=test;
  by pcode numero;
run;

 /*** Choix de l'indice de dfavorisation si plus d'un indice par code postal ***/
data test1;
  set test;
  by pcode numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1; 
    else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propcp_RLS/100);
	if nba < cumul then do; 
      find=1; output; 
    end;
  end;
run; 

/*** CLSC ***/
proc sql;
  create table test as
  select fichin.pcode, fichin.numero, fichin.nba, pccfdups.first, pccfdups.quartmatCLSC, pccfdups.quartsocCLSC, 
         pccfdups.propcp_CLSC, pccfdups.nuniq, pccfdups.CLSC 
  from fichin, pccfdups
  where fichin.pcode=pccfdups.pcode
  order by numero,nuniq;
quit;

proc sort data=test;
  by pcode numero;
run;

 /*** Choix de l'indice de dfavorisation si plus d'un indice par code postal ***/
data test2;
  set test;
  by pcode numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1; 
    else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propcp_CLSC/100);
	if nba < cumul then do; 
      find=1; output; 
    end;
  end;
run; 

/*** Qubec et RSS ***/
proc sql;
  create table test as
  select fichin.pcode, fichin.numero, fichin.nba, pccfdups.first, pccfdups.quintmat, pccfdups.quintsoc, pccfdups.quintmatRSS, pccfdups.quintsocRSS, 
         pccfdups.quartmatRSS, pccfdups.quartsocRSS, pccfdups.propcp, pccfdups.nuniq, pccfdups.zone, pccfdups.RSS, pccfdups.RTS, pccfdups.TGN_AD, pccfdups.discord,
		 pccfdups.quintmatRTS, pccfdups.quintsocRTS, pccfdups.quartmatRTS, pccfdups.quartsocRTS
  from fichin, pccfdups
  where fichin.pcode=pccfdups.pcode
  order by numero,nuniq;
quit;

proc sort data=test;
  by pcode numero;
run;

/*** Choix de l'indice de dfavorisation si plus d'un indice par code postal ***/
data test3;
  set test;
  by pcode numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1; 
    else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propcp/100);
	if nba < cumul then do; 
      find=1; output; 
    end;
  end;
run; 

proc sort data=test1;
  by numero;
run;

proc sort data=test2;
  by numero;
run;

proc sort data=test3;
  by numero;
run;

proc sort data=nonapp4;
  by numero;
run; 

data appcp3 nonapp5;
  merge nonapp4(in=in1) test1(in=in2a) test2(in=in2b) test3(in=in2c);
  by numero;
  if in1*in2a*in2b*in2c then output appcp3;
  else if in1 then output nonapp5;
run;



data tout;
  set %if &munic ne 0 %then %do; appmunic1(in=in1) appmunic2(in=in2) appmunic3(in=in3) %end; 
                                 appcp1(in=in4) appcp2(in=in5) appcp3(in=in6) nonapp5(in=in7);
  if in7 then direct=0;
  if in5 then do; 
    direct=1;
  end;

  %if &munic ne 0 %then %do; 
     if in1 then base=1;
     else if in2 then do; 
       direct=1;base=2;
     end;
	 else if in3 then base=3;
   %end;
   if in4 then base=1;
   else if in5 then base=2;
   else if in6 then base=3;

   if in7=1 then do;quintmat = 0;quintsoc = 0;quartmatRLS=0;quartsocRLS=0;quartmatCLSC=0;quartsocCLSC=0;quartmatRSS=0;quartsocRSS=0;quintmatRSS=0; quintsocRSS=0;
                    zone = 0;quartmatRTS=0;quartsocRTS=0;quintmatRTS=0; quintsocRTS=0;
   end;
   keep numero base quintmat quintsoc quartmatRSS quartsocRSS quintmatRSS quintsocRSS quartmatRTS quartsocRTS quintmatRTS quintsocRTS quartmatRLS quartsocRLS 
        quartmatCLSC quartsocCLSC zone rss rls rts clsc TGN_AD discord;
run;

proc sort; 
  by numero; 
run; 

data fichin;
  set &in;
  numero=_n_;
run;

data tout;
  retain TGN_AD discord zone RSS RTS RLS CLSC quintmat quintsoc quintmatRSS quintsocRSS quartmatRSS quartsocRSS quintmatRTS quintsocRTS quartmatRTS quartsocRTS
         quartmatRLS quartsocRLS quartmatCLSC quartsocCLSC;
  set tout;
run;

data &out;
  merge fichin tout;
  by numero;
  DATE = '20150902';
  *format groupe fgroupe. base fbase.;
  drop numero;
run;

proc datasets library=work;
  delete corr2 wcf2 pccfuniq2 pccfdups2 test test1 test2 test3 fichin tout appmunic1 appmunic2 appmunic3 appmunic4 appcp1 appcp2 appcp3 appcp4 nonapp1 nonapp2 
         nonapp3 nonapp4 nonapp5 nonapp6 nonapp7 pccfdup2; 
quit;
%mend assignation2011;

%macro presence(var=);
%global presence;
%let dsid=%sysfunc(open(fichin,i));
    %if &dsid>0 %then
       %do;
	     %let presence=%sysfunc(varnum(&dsid,&var));
		 /*** la macrovariable presence = 0 si la variable est absente du fichier ****/
	   %end;
%put &presence;
%let rc = %sysfunc(close(&dsid));
%mend presence;


/***** Exemple *****/
%let in = deathmaster;
%let pcode = CPRESDCD;
%let munic = CMUNRESD;
%let out = fichier_assigne;

%assignation2011(in=&in,pcode=&pcode,munic=&munic,out=&out);
